# $Id: 00.load.t,v 1.2 2003/10/28 17:06:59 andy Exp $

use strict;
use Test::More tests => 11;

use_ok( "WDDX::Array" );
use_ok( "WDDX::Binary" );
use_ok( "WDDX::Boolean" );
use_ok( "WDDX::Datetime" );
use_ok( "WDDX::Null" );
use_ok( "WDDX::Number" );
use_ok( "WDDX::Parser" );
use_ok( "WDDX::Recordset" );
use_ok( "WDDX::String" );
use_ok( "WDDX::Struct" );
use_ok( "WDDX" );
